/*
 * TismTool  Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * OSAL_MutexSem.c
 * version 3
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_MutexSem.h"
#include <windows.h>

#define ASSERT_MUTEX(h) assert((0<(h))&&((h)<=OSAL_NOF_MUTEXSEMS)&&(MutexSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool         bOccupied;
  CRITICAL_SECTION  cs;
} OSAL_MutexSem;

static CRITICAL_SECTION MutexCs;
static OSAL_MutexSem    MutexSems[ OSAL_NOF_MUTEXSEMS ];

void OSAL_MutexSemInitialise( void )
{
  memset( MutexSems, 0, sizeof(MutexSems) );
  InitializeCriticalSection( &MutexCs );
}

void OSAL_MutexSemTerminate( void )
{
  DeleteCriticalSection( &MutexCs );
}

OSAL_Handle OSAL_MutexSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;

  EnterCriticalSection( &MutexCs );
  for ( i=0; i<OSAL_NOF_MUTEXSEMS; i++ )
  {
    if ( ! MutexSems[ i ].bOccupied )
    {
      InitializeCriticalSection( &MutexSems[ i ].cs );
      MutexSems[ i ].bOccupied = TRUE;
      h = i+1;
      break;
      /****/
    }
  }
  LeaveCriticalSection( &MutexCs );

  return h;
}

void OSAL_MutexSemLock( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  EnterCriticalSection( &MutexSems[ h-1 ].cs );
}

void OSAL_MutexSemUnlock( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  LeaveCriticalSection( &MutexSems[ h-1 ].cs );
}

void OSAL_MutexSemReset( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  DeleteCriticalSection( &MutexSems[ h-1 ].cs );
  InitializeCriticalSection( &MutexSems[ h-1 ].cs );
}

void OSAL_MutexSemDelete( OSAL_Handle h )
{
  ASSERT_MUTEX(h);
  EnterCriticalSection( &MutexCs );

  DeleteCriticalSection( &MutexSems[ h-1 ].cs );
  MutexSems[ h-1 ].bOccupied = FALSE;

  LeaveCriticalSection( &MutexCs );
}
