/*
 * TismTool  Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * OSAL_RzVousSem.c
 * version 3
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_RzVousSem.h"
#include <windows.h>

#define ASSERT_RZVOUS(h) assert((0<(h))&&((h)<=OSAL_NOF_RZVOUSSEMS)&&(RzVousSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool   bOccupied;
  HANDLE      Handle;
} OSAL_RzVousSem;

static CRITICAL_SECTION RzVousCs;
static OSAL_RzVousSem   RzVousSems[ OSAL_NOF_RZVOUSSEMS ];

void OSAL_RzVousSemInitialise( void )
{
  memset( RzVousSems, 0, sizeof(RzVousSems) );
  InitializeCriticalSection( &RzVousCs );
}

void OSAL_RzVousSemTerminate( void )
{
  DeleteCriticalSection( &RzVousCs );
}

OSAL_Handle OSAL_RzVousSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;

  EnterCriticalSection( &RzVousCs );
  for ( i=0; i<OSAL_NOF_RZVOUSSEMS; i++ )
  {
    if ( ! RzVousSems[ i ].bOccupied )
    {
      RzVousSems[ i ].Handle = CreateSemaphore( NULL, 0, 1, NULL );
      if ( NULL != RzVousSems[ i ].Handle )
      {
        RzVousSems[ i ].bOccupied = TRUE;
        h = i+1;
      }
      break;
      /****/
    }
  }
  LeaveCriticalSection( &RzVousCs );

  return h;
}

void OSAL_RzVousSemWait( OSAL_Handle h )
{
  ASSERT_RZVOUS(h);
  WaitForSingleObject( RzVousSems[ h-1 ].Handle, INFINITE );
}

void OSAL_RzVousSemRelease( OSAL_Handle h )
{
  ASSERT_RZVOUS(h);
  ReleaseSemaphore( RzVousSems[ h-1 ].Handle, 1, NULL );
}

void OSAL_RzVousSemReset( OSAL_Handle h )
{
  ASSERT_RZVOUS(h);
  CloseHandle( RzVousSems[ h-1 ].Handle );
  RzVousSems[ h-1 ].Handle = CreateSemaphore( NULL, 0, 1, NULL );
  assert( NULL != RzVousSems[ h-1 ].Handle );
}

void OSAL_RzVousSemDelete( OSAL_Handle h )
{
  ASSERT_RZVOUS(h);
  EnterCriticalSection( &RzVousCs );

  CloseHandle( RzVousSems[ h-1 ].Handle );
  RzVousSems[ h-1 ].bOccupied = FALSE;
  RzVousSems[ h-1 ].Handle = NULL;

  LeaveCriticalSection( &RzVousCs );
}
