/*
 * TismTool  Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * OSAL_Thread.c
 * version 4
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_Thread.h"
#include <windows.h>

#define ASSERT_THREAD(h) assert((0<(h))&&((h)<=OSAL_NOF_THREADS+1)&&(Threads[(h)-1].bOccupied))

static CRITICAL_SECTION ThreadCs;
static OSAL_Thread      Threads[ OSAL_NOF_THREADS+1 ];

void OSAL_ThreadInitialise( void )
{
  memset( Threads, 0, sizeof(Threads) );
  InitializeCriticalSection( &ThreadCs );
}

void OSAL_ThreadTerminate( void )
{
  DeleteCriticalSection( &ThreadCs );
}

OSAL_Handle OSAL_ThreadCreate( void (*pFct)(void*), void* pArg )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;
  DWORD       ID;

  EnterCriticalSection( &ThreadCs );
  for ( i=1; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( ! Threads[ i ].bOccupied )
    {
      LPTHREAD_START_ROUTINE sr = (LPTHREAD_START_ROUTINE) pFct;

      Threads[ i ].Handle = CreateThread( 
            NULL,           /* default security attributes   */
            OSAL_STACKSIZE, /* 0 = use default stack size    */
            sr,             /* thread function name          */
            pArg,           /* argument to thread function   */
            0,              /* use default creation flags    */
            &ID );          /* returns the thread identifier */

      if ( NULL != Threads[ i ].Handle )
      {
        Threads[ i ].bOccupied = TRUE;
        Threads[ i ].IDThread  = ID;
        h = i + 1;
      }
      break;
      /****/
    }
  }
  LeaveCriticalSection( &ThreadCs );

  return h;
}

OSAL_Handle OSAL_SetMainThread( void )
{
  HANDLE Handle = GetCurrentThread();
  DWORD  ID = GetCurrentThreadId();

  Threads[ 0 ].bOccupied = TRUE;
  Threads[ 0 ].Handle    = Handle;
  Threads[ 0 ].IDThread  = ID;

  return 1;
}

OSAL_Thread* OSAL_GetCurrentThread( void )
{
  OSAL_Thread*  pThread = NULL;
  OSAL_Handle   h = OSAL_GetCurrentThreadHandle();

  if ( h != OSAL_HANDLE_INVALID )
  {
    pThread = &Threads[ h-1 ];
  }

  return pThread;
}

OSAL_Handle OSAL_GetCurrentThreadHandle( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  DWORD       ID = GetCurrentThreadId();
  int         i;

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( ( Threads[ i ].bOccupied      )
      && ( Threads[ i ].IDThread == ID ) )
    {
      h = i + 1;
      break;
      /****/
    }
  }

  return h;
}

void OSAL_ThreadDelete( OSAL_Handle h )
{
  ASSERT_THREAD(h);
  EnterCriticalSection( &ThreadCs );

  CloseHandle( Threads[ h-1 ].Handle );
  Threads[ h-1 ].bOccupied = FALSE;
  Threads[ h-1 ].Handle = NULL;

  LeaveCriticalSection( &ThreadCs );
}

void OSAL_ThreadSetAffinity( OSAL_Handle h, unsigned long Mask )
{
  DWORD_PTR PreviousMask;

  ASSERT_THREAD(h);

  PreviousMask = SetThreadAffinityMask( Threads[ h-1 ].Handle, (DWORD_PTR) Mask );
  assert( PreviousMask != 0 );
}

void OSAL_ThreadSleep( int MilliSeconds )
{
  Sleep( MilliSeconds );
}

void OSAL_ThreadYield( void )
{
  SwitchToThread();
}
