/*
 * TismTool  Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * OSAL_TriggerSem.c
 * version 3
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_TriggerSem.h"
#include <windows.h>

#define ASSERT_TRIGGER(h) assert((0<(h))&&((h)<=OSAL_NOF_TRIGGERSEMS)&&(TriggerSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool   bOccupied;
  HANDLE      Handle;
} OSAL_TriggerSem;

static CRITICAL_SECTION TriggerCs;
static OSAL_TriggerSem  TriggerSems[ OSAL_NOF_TRIGGERSEMS ];

void OSAL_TriggerSemInitialise( void )
{
  memset( TriggerSems, 0, sizeof(TriggerSems) );
  InitializeCriticalSection( &TriggerCs );
}

void OSAL_TriggerSemTerminate( void )
{
  DeleteCriticalSection( &TriggerCs );
}

OSAL_Handle OSAL_TriggerSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;

  EnterCriticalSection( &TriggerCs );
  for ( i=0; i<OSAL_NOF_TRIGGERSEMS; i++ )
  {
    if ( ! TriggerSems[ i ].bOccupied )
    {
      TriggerSems[ i ].Handle = CreateSemaphore( NULL, 0, 1, NULL );
      if ( NULL != TriggerSems[ i ].Handle )
      {
        TriggerSems[ i ].bOccupied = TRUE;
        h = i+1;
      }
      break;
      /****/
    }
  }
  LeaveCriticalSection( &TriggerCs );

  return h;
}

void OSAL_TriggerSemWait( OSAL_Handle h )
{
  ASSERT_TRIGGER(h);
  WaitForSingleObject( TriggerSems[ h-1 ].Handle, INFINITE );
}

void OSAL_TriggerSemRelease( OSAL_Handle h )
{
  ASSERT_TRIGGER(h);
  ReleaseSemaphore( TriggerSems[ h-1 ].Handle, 1, NULL );
}

void OSAL_TriggerSemReset( OSAL_Handle h )
{
  ASSERT_TRIGGER(h);
  CloseHandle( TriggerSems[ h-1 ].Handle );
  TriggerSems[ h-1 ].Handle = CreateSemaphore( NULL, 0, 1, NULL );
  assert( NULL != TriggerSems[ h-1 ].Handle );
}

void OSAL_TriggerSemDelete( OSAL_Handle h )
{
  ASSERT_TRIGGER(h);
  EnterCriticalSection( &TriggerCs );

  CloseHandle( TriggerSems[ h-1 ].Handle );
  TriggerSems[ h-1 ].bOccupied = FALSE;
  TriggerSems[ h-1 ].Handle = NULL;

  LeaveCriticalSection( &TriggerCs );
}
