/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmClass.c
 * version 3
 */
#include <string.h>
#include <stdio.h>
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmClass.h"
#include "TsmException.h"
#include "TsmTrigger.h"
#include "TsmArray.h"
#include "TsmProxyUser.h"
#include "TsmInterfacePort.h"
#include "TsmHandler.h"

TsmClass* TsmClass_Ctor( void )
{
  TsmClass* this = (TsmClass*) Tsm_Malloc( sizeof(TsmClass) );
  TsmClass_Ftor( this );
  return this;
}

void TsmClass_Ftor( TsmClass* this )
{
  TsmBase_Ftor( (TsmBase*) this );
  this->Class = "TsmClass";
  this->mNrFsms = 0;
  this->oFsms = NULL;
  this->IsMock = FALSE;
  this->SnapState = NULL;
  this->SnapCall = NULL;
  this->SeqId = 0;

  this->Gtor = (Tsm_pfObj) TsmClass_Gtor;
  this->SetProxyUser = TsmClass_SetProxyUser;
  this->GetProvidedInterface = TsmClass_GetProvidedInterface;
  this->SetRequiredInterface = TsmClass_SetRequiredInterface;
  this->ExecuteTrigger = TsmClass_ExecuteTrigger;
  this->ExecuteNative = TsmClass_ExecuteNative;
  this->TsmInitialise = TsmClass_TsmInitialise;
  this->TsmConnect = TsmClass_TsmConnect;
  this->TsmPrepare = TsmClass_TsmPrepare;
  this->TsmRun = TsmClass_TsmRun;
  this->TsmHalt = TsmClass_TsmHalt;
  this->TsmCease = TsmClass_TsmCease;
  this->TsmDisconnect = TsmClass_TsmDisconnect;
  this->TsmTerminate = TsmClass_TsmTerminate;
  this->TsmMention = TsmClass_TsmMention;
  this->TsmReset1 = TsmClass_TsmReset1;
  this->TsmReset2 = TsmClass_TsmReset2;
  this->TsmSnap = TsmClass_TsmSnap;
  this->TsmCheckVisit = TsmClass_TsmCheckVisit;
  this->IsTsmProxy = TsmClass_IsTsmProxy;
  this->IsTsmProxyUser = TsmClass_IsTsmProxyUser;

  TsmNativeLists_Initialise( &this->Natives );
}

void TsmClass_Gtor( TsmClass* this )
{
  int i;

  for ( i=0; i<this->mNrFsms; i++ )
  {
    this->oFsms[i]->Dtor( (TsmObject*) this->oFsms[i] );
  }
  TsmBase_Gtor( (TsmBase*) this );
}

void TsmClass_SetProxyUser( TsmClass* this, TsmClass* oProxyUser )
{
  /* ignore */
}

TsmInterfacePort* TsmClass_GetProvidedInterface( TsmClass* this, char* sPortName )
{
  TsmStr sb;
  
  sb = Tsm_Malloc( TSM_NCH_FULLNAME );
  sprintf( sb, "9500: GetProvidedInterface should be overriden for '%s' @ '", sPortName );
  this->FullName( (TsmBase*) this, sb+strlen(sb) );
  strcat( sb, "'" );
  TsmException_Quit( sb );
  Tsm_Free( sb );

  return NULL;
}

void TsmClass_SetRequiredInterface( TsmClass* this, char* sPortName, TsmInterfacePort* oProvidedInterface )
{
  TsmStr sb;
  
  sb = Tsm_Malloc( TSM_NCH_FULLNAME );
  sprintf( sb, "9510: SetRequiredInterface should be overriden for '%s' @ '", sPortName );
  this->FullName( (TsmBase*) this, sb+strlen(sb) );
  strcat( sb, "'" );
  TsmException_Quit( sb );
  Tsm_Free( sb );
}

void TsmClass_ExecuteTrigger( TsmClass* this, TsmTrigger* oTrigger )
{
}

void TsmClass_ExecuteNative( TsmClass* this, int iCmd )
{
  TsmTrigger lTrigger;

  TsmTrigger_Ftor( &lTrigger, iCmd );
  lTrigger.PortId = TsmClass_NativePortId;
  this->ExecuteTrigger( this, &lTrigger );
}

void TsmClass_TsmInitialise( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmConnect( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmPrepare( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmRun( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmHalt( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmCease( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmDisconnect( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmTerminate( TsmClass* this, void* oObject )
{
}

void TsmClass_TsmMention( TsmClass* this, void* oObject )
{
  TsmArray* ClassList = (TsmArray*) oObject;
  char*     sc;
  
  this->ClassListIndex = TsmArray_Add( ClassList, this );

  sc = Tsm_Malloc( TSM_NCH_FULLNAME );
  strcpy( sc, "Class," );
  this->FullName( (TsmBase*) this, sc+strlen(sc) );
  strcat( sc, "," );
  this->SeqId = Tsm_SeqClass( sc );
  Tsm_Free( sc );
}

void TsmClass_TsmReset1( TsmClass* this )
{
  TsmHandler  h;
  int         i;

  for ( i=0; i<this->mNrFsms; i++ )
  {
    TsmHandler_Ftor1( &h, this );
    this->oFsms[i]->TsmReset1( this->oFsms[i], &h );
  }
  Tsm_Free( this->SnapState ); /* Invalidate SnapState */
  this->SnapState = NULL;
}

void TsmClass_TsmReset2( TsmClass* this )
{
  TsmHandler  h;
  int         i;

  for ( i=0; i<this->mNrFsms; i++ )
  {
    TsmHandler_Ftor1( &h, this );
    this->oFsms[i]->TsmReset2( this->oFsms[i], &h );
  }
}

void TsmClass_TsmSnap( TsmClass* this, TsmStr sout )
{
  if ( this->SnapState == NULL )
  {
    int   i;
    char* sb;

    sb = Tsm_Malloc( TSM_NCH_FULLNAME );
    sprintf( sb, "{%s", this->TsmName );
    for ( i=0; i<this->mNrFsms; i++ )
    {
      strcat( sb, "/" );
      this->oFsms[i]->TsmSnap( (TsmState*) this->oFsms[i], sb+strlen(sb) );
    }
    strcat( sb, "}" );
    
    this->SnapState = Tsm_Restring( sb );
  }
  strcpy( sout, this->SnapState );
  if ( this->SnapCall != NULL )
  {
    strcat( sout, this->SnapCall );
  }
}

void TsmClass_TsmCheckVisit( TsmClass* this )
{
  int i;

  for ( i=0; i<this->mNrFsms; i++ )
  {
    this->oFsms[i]->TsmCheckVisit( (TsmState*) this->oFsms[i] );
  }
}

TsmBool TsmClass_IsTsmProxy( TsmClass* this )
{
  return FALSE;
}

TsmBool TsmClass_IsTsmProxyUser( TsmClass* this )
{
  return FALSE;
}
