/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmComponent.c
 * version 1
 */
#include <stdio.h>
#include <string.h>
#include "TsmClass.h"
#include "TsmComponent.h"
#include "TsmGen.h"
#include "Tsm.h"
#include "TsmInterfacePort.h"

TsmComponent* TsmComponent_Ctor( void )
{
  TsmComponent* this = (TsmComponent*) Tsm_Malloc( sizeof(TsmComponent) );
  TsmComponent_Ftor( this );
  return this;
}

void TsmComponent_Ftor( TsmComponent* this )
{
  TsmClass_Ftor( (TsmClass*) this );
  this->Class = "TsmComponent";

  this->mNrClasses = 0;
  this->mClasses = NULL;
  this->mNrComponents = 0;
  this->mComponents = NULL;

  this->Gtor = (Tsm_pfObj) TsmComponent_Gtor;
  this->IsActive = TsmComponent_IsActive;
  this->ThreadId = TsmComponent_ThreadId;
  this->TsmInitialiseAll = TsmComponent_TsmInitialiseAll;
  this->TsmConnectAll = TsmComponent_TsmConnectAll;
  this->TsmPrepareAll = TsmComponent_TsmPrepareAll;
  this->TsmRunAll = TsmComponent_TsmRunAll;
  this->TsmHaltAll = TsmComponent_TsmHaltAll;
  this->TsmCeaseAll = TsmComponent_TsmCeaseAll;
  this->TsmDisconnectAll = TsmComponent_TsmDisconnectAll;
  this->TsmTerminateAll = TsmComponent_TsmTerminateAll;
  this->TsmMentionAll = TsmComponent_TsmMentionAll;
  this->TsmCheckVisitAll = TsmComponent_TsmCheckVisitAll;
  this->DoAssembly = TsmComponent_DoAssembly;
}

void TsmComponent_Gtor( TsmComponent* this )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->Dtor( (TsmObject*) this->mClasses[i] );
  }

  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->Dtor( (TsmObject*) this->mComponents[i] );
  }

  TsmClass_Gtor( (TsmClass*) this );
}

TsmBool TsmComponent_IsActive( TsmComponent* this )
{
  return FALSE;
}

void TsmComponent_ThreadId( TsmComponent* this, TsmStr sa )
{
  *sa = '\0';
}

void TsmComponent_TsmInitialiseAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmInitialise( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmInitialiseAll( this->mComponents[i], oObject );
  }

  this->TsmInitialise( (TsmClass*) this, oObject );
}

void TsmComponent_TsmConnectAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmConnect( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmConnectAll( this->mComponents[i], oObject );
  }

  this->TsmConnect( (TsmClass*) this, oObject );
}

void TsmComponent_TsmPrepareAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmPrepare( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmPrepareAll( this->mComponents[i], oObject );
  }

  this->TsmPrepare( (TsmClass*) this, oObject );
}

void TsmComponent_TsmRunAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmRun( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmRunAll( this->mComponents[i], oObject );
  }

  this->TsmRun( (TsmClass*) this, oObject );
}

void TsmComponent_TsmHaltAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmHalt( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmHaltAll( this->mComponents[i], oObject );
  }

  this->TsmHalt( (TsmClass*) this, oObject );
}

void TsmComponent_TsmCeaseAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmCease( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmCeaseAll( this->mComponents[i], oObject );
  }

  this->TsmCease( (TsmClass*) this, oObject );
}

void TsmComponent_TsmDisconnectAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmDisconnect( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmDisconnectAll( this->mComponents[i], oObject );
  }

  this->TsmDisconnect( (TsmClass*) this, oObject );
}

void TsmComponent_TsmTerminateAll( TsmComponent* this, void* oObject )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmTerminate( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmTerminateAll( this->mComponents[i], oObject );
  }

  this->TsmTerminate( (TsmClass*) this, oObject );
}

void TsmComponent_TsmMentionAll( TsmComponent* this, void* oObject )
{
  char* sb;
  int   i;

  sb = Tsm_Malloc( TSM_NCH_FULLNAME );
  sprintf( sb, "Comp," );
  this->FullName( (TsmBase*) this, sb+strlen(sb) );
  this->ThreadId( this, sb+strlen(sb) );
  Tsm_Seq( sb );
  Tsm_Free( sb );

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmMention( this->mClasses[i], oObject );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmMentionAll( this->mComponents[i], oObject );
  }
}

void TsmComponent_TsmCheckVisitAll( TsmComponent* this )
{
  int i;

  for ( i=0; i<this->mNrClasses; i++ )
  {
    this->mClasses[i]->TsmCheckVisit( this->mClasses[i] );
  }
  for ( i=0; i<this->mNrComponents; i++ )
  {
    this->mComponents[i]->TsmCheckVisitAll( this->mComponents[i] );
  }
}

void TsmComponent_DoAssembly( TsmComponent* this,
                              TsmClass* Client, char* sRequired,
                              TsmClass* Server, char* sProvided )
{
  TsmInterfacePort* lPort;

  lPort = Server->GetProvidedInterface( Server, sProvided );
  Client->SetRequiredInterface( Client, sRequired, lPort );
}
