/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmExecComponent.c
 * version 3
 */
#include <string.h>
#include "TsmComponent.h"
#include "TsmExecComponent.h"
#include "TsmGen.h"
#include "TsmIntQueue.h"
#include "TsmSupervisor.h"
#include "OSAL_Thread.h"

TsmExecComponent* TsmExecComponent_Ctor( void )
{
  TsmExecComponent* this = (TsmExecComponent*) Tsm_Malloc( sizeof(TsmExecComponent) );
  TsmExecComponent_Ftor( this );
  return this;
}

void TsmExecComponent_Ftor( TsmExecComponent* this )
{
  TsmComponent_Ftor( (TsmComponent*) this );
  this->Class = "TsmExecComponent";

  this->Activate = TsmExecComponent_Activate;
  this->Execute = TsmExecComponent_Execute;
  this->Deactivate = TsmExecComponent_Deactivate;
}

void TsmExecComponent_Activate( TsmExecComponent* this )
{
  TsmIntQueue* lIntQueue;

  TsmSupervisor_Initialise();
  lIntQueue = TsmSupervisor_GetIntQueue();

  /*
   * Force all threads running on the first cpu.
   */
  OSAL_ThreadSetAffinity( OSAL_GetCurrentThreadHandle(), 1 );

  this->TsmInitialiseAll( (TsmComponent*) this, NULL );
  this->TsmConnectAll( (TsmComponent*) this, lIntQueue );
  this->TsmPrepareAll( (TsmComponent*) this, NULL );
  this->TsmRunAll( (TsmComponent*) this, NULL );
}

void TsmExecComponent_Execute( TsmExecComponent* this )
{
  this->TsmMentionAll( (TsmComponent*) this, TsmSupervisor_GetClasses() );

  TsmSupervisor_Verify();

  this->TsmCheckVisitAll( (TsmComponent*) this );

  TsmSupervisor_Summary();
  TsmSupervisor_Cleanup();
}

void TsmExecComponent_Deactivate( TsmExecComponent* this )
{
  this->TsmHaltAll( (TsmComponent*) this, NULL );
  this->TsmCeaseAll( (TsmComponent*) this, NULL );
  this->TsmDisconnectAll( (TsmComponent*) this, NULL );
  this->TsmTerminateAll( (TsmComponent*) this, NULL );

  TsmSupervisor_Terminate();
}
