/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * Tsm.c
 * version 2
 */
#include <stdio.h>
#include "Tsm.h"
#include "OSAL_Gen.h"
#include "OSAL_MutexSem.h"

static void Tsm_FreeTismFile( char* sFile, char* sName );

static TsmBool    mbLogConsole  = FALSE;
static TsmBool    mbLogFile     = TRUE;
static int        mLogLevel     = 1;
static TsmBool    mbSeqFile     = TRUE;
static int        mSeqClassId   = 0;

static FILE*        mLogWriter = NULL;
static OSAL_Handle  mLogLockMutex = OSAL_HANDLE_INVALID;
static FILE*        mSeqWriter = NULL;
static OSAL_Handle  mSeqLockMutex = OSAL_HANDLE_INVALID;

void Tsm_SetLogConsole( TsmBool bLogConsole )
{
  mbLogConsole = bLogConsole;
}

void Tsm_SetLogFile( TsmBool bLogFile )
{
  mbLogFile = bLogFile;
}

void Tsm_SetLogLevel( int iLevel )
{
  mLogLevel = iLevel;
}

void Tsm_SetSeqFile( TsmBool bSeqFile )
{
  mbSeqFile = bSeqFile;
}

void Tsm_Assert( TsmBool bCondition, char* msg, char* file, int line )
{
  if ( ! bCondition )
  {
    char sc[ 1024 ];
    sprintf( sc, "ASSERT @ %s#%d : %s\n", file, line, msg );
    Tsm_Log( sc );
  }
}

void Tsm_Open( void )
{
  char s[ 20 ];

  if ( ( mbLogConsole != FALSE ) || ( mbLogFile != FALSE ) )
  {
    mLogLockMutex = OSAL_MutexSemCreate();
    if ( mLogLockMutex == OSAL_HANDLE_INVALID )
    {
      printf( "ERROR: OSAL_MutexSemCreate" );
      mbLogConsole = mbLogFile = FALSE;
    }
  }

  if ( mbLogFile != FALSE )
  {
    Tsm_FreeTismFile( s, "Log" );
    mLogWriter = fopen( s, "w" );
    if ( mLogWriter == NULL )
    {
      printf( "ERROR: opening file: %s\n", s );
      mbLogFile = FALSE;
    }
  }

  if ( mbSeqFile != FALSE )
  {
    mSeqLockMutex = OSAL_MutexSemCreate();
    if ( mSeqLockMutex == OSAL_HANDLE_INVALID )
    {
      printf( "ERROR: OSAL_MutexSemCreate" );
      mbSeqFile = FALSE;
    }
    else
    {
      Tsm_FreeTismFile( s, "Seq" );
      mSeqWriter = fopen( s, "w" );
      if ( mSeqWriter == NULL )
      {
        printf( "ERROR: opening file: %s\n", s );
        mbSeqFile = FALSE;
      }
      else
      {
        Tsm_Seq( "TismTool, Sequence Diagrams" );
      }
    }
  }
}

void Tsm_Log0( char* s )
{
  if ( mbLogFile != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    fprintf( mLogWriter, "%s", s );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }

  if ( mbLogConsole != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    printf( "%s", s );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }
}

void Tsm_Log( char* s )
{
  if ( mbLogFile != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    fprintf( mLogWriter, "%s\n", s );
    fflush( mLogWriter );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }

  if ( mbLogConsole != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    printf( "%s\n", s );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }
}

void Tsm_Log2( char* head, char* s )
{
  if ( mbLogFile != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    fprintf( mLogWriter, "%s%s\n", head, s );
    fflush( mLogWriter );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }

  if ( mbLogConsole != FALSE )
  {
    OSAL_MutexSemLock( mLogLockMutex );
    printf( "%s%s\n", head, s );
    OSAL_MutexSemUnlock( mLogLockMutex );
  }
}

int Tsm_SeqClass( char* s )
{
  int Id = -1;

  if ( mbSeqFile != FALSE )
  {
    OSAL_MutexSemLock( mSeqLockMutex );
    Id = ++mSeqClassId;
    OSAL_MutexSemUnlock( mSeqLockMutex );
  }

  Tsm_Seq2( s, Id );

  return Id;
}

void Tsm_Seq0( char* s )
{
  if ( mbSeqFile != FALSE )
  {
    OSAL_MutexSemLock( mSeqLockMutex );
    fprintf( mSeqWriter, "%s", s );
    OSAL_MutexSemUnlock( mSeqLockMutex );
  }
}

void Tsm_Seq( char* s )
{
  if ( mbSeqFile != FALSE )
  {
    OSAL_MutexSemLock( mSeqLockMutex );
    fprintf( mSeqWriter, "%s\n", s );
    fflush( mSeqWriter );
    OSAL_MutexSemUnlock( mSeqLockMutex );
  }
}

void Tsm_Seq2( char* s, int n )
{
  if ( mbSeqFile != FALSE )
  {
    OSAL_MutexSemLock( mSeqLockMutex );
    fprintf( mSeqWriter, "%s%d\n", s, n );
    fflush( mSeqWriter );
    OSAL_MutexSemUnlock( mSeqLockMutex );
  }
}

void Tsm_Lgl0( int iLevel, char* s )
{
  if ( iLevel <= mLogLevel )
  {
    Tsm_Log0( s );
  }
}

void Tsm_Lgl( int iLevel, char* s )
{
  if ( iLevel <= mLogLevel )
  {
    Tsm_Log( s );
  }
}

void Tsm_ShowException( char* s )
{
  Tsm_Lgl( 1, s );
}

void Tsm_Close( void )
{
  if ( mLogWriter != NULL )
  {
    fclose( mLogWriter );
    mLogWriter = NULL;
  }

  if ( mLogLockMutex != OSAL_HANDLE_INVALID )
  {
    OSAL_MutexSemDelete( mLogLockMutex );
    mLogLockMutex = OSAL_HANDLE_INVALID;
  }

  if ( mSeqWriter != NULL )
  {
    fclose( mSeqWriter );
    mSeqWriter = NULL;
  }

  if ( mSeqLockMutex != OSAL_HANDLE_INVALID )
  {
    OSAL_MutexSemDelete( mSeqLockMutex );
    mSeqLockMutex = OSAL_HANDLE_INVALID;
  }
}

char* StackTracestrings( void )
{
  return "NoTrace";
}

void ReportBox( char* s )
{
  printf( "Report: %s\n", s );
}

static void Tsm_FreeTismFile( char* sFile, char* sName )
{
  FILE*   fi;
  int     ix = 0;

  for ( ; ; )
  {
    ix++;
    sprintf( sFile, "Tism%s_%04d.txt", sName, ix );
    fi = fopen( sFile, "r" );
    if ( fi == NULL )
    {
      break;
    }
    fclose( fi );
  }
}
