/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmList.c
 * version 1
 */
#include <stdlib.h>
#include "TsmList.h"
#include "TsmGen.h"
#include "TsmException.h"

void TsmList_Initialise( TsmList* this, int Size, TsmListItem* Items )
{
  int i;

  for ( i=0; i<Size-1; i++ )
  {
    Items[i].pNext = &Items[i+1];
  }
  Items[Size-1].pNext = NULL;
  this->pFree = &Items[0];
  this->pOccupied = NULL;
}

void TsmList_Add( TsmList* this, TsmListValue Value )
{
  TsmListItem* p = this->pFree;

  if ( NULL == p )
  {
    TsmException_Quit( "TsmList_Add: full, increase size" );
  }

  this->pFree = this->pFree->pNext;
  p->pNext = this->pOccupied;
  this->pOccupied = p;
  p->Value = Value;
}

TsmBool TsmList_Remove( TsmList* this, TsmListValue Value )
{
  TsmBool      bFound = FALSE;
  TsmListItem* p = this->pOccupied;
  TsmListItem* pPrev = NULL;

  while ( NULL != p )
  {
    if ( p->Value == Value )
    {
      bFound = TRUE;
      break;
      /****/
    }
    pPrev = p;
    p = p->pNext;
  }

  if ( bFound )
  {
    if ( NULL == pPrev )
    {
      /* p = this->pOccupied; */
      this->pOccupied = p->pNext;
    }
    else
    {
      pPrev->pNext = p->pNext;
    }
    p->pNext = this->pFree;
    this->pFree = p;
  }

  return bFound;
}

void TsmList_Clear( TsmList* this )
{
  TsmListItem* pPrev = NULL;
  TsmListItem* p = this->pOccupied;

  while ( NULL != p )
  {
    pPrev = p;
    p = p->pNext;
  }

  if ( NULL != pPrev )
  {
    pPrev->pNext = this->pFree;
    this->pFree = this->pOccupied;
    this->pOccupied = NULL;
  }
}
