/*
 * TismTool  Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmMapString.c
 * version 4
 */
#include <stdio.h>
#include <string.h>
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmArray.h"
#include "TsmMapString.h"
  
typedef unsigned int TsmMSHash;

#define TsmMSFill_NOK ((unsigned char) 0)
#define TsmMSFill_OK  ((unsigned char) 1)
#define TsmMSFill_OVW ((unsigned char) 2)

typedef struct TsmMSOvwEntryStruct
{
  TsmMSHash         Hash;
  char*             Key;
  TsmMapStringValue Value;
} TsmMSOvwEntry;

static TsmMSHash TsmMapString_Hash( TsmMapString* this, char* Key )
{
  TsmMSHash Hash = 0;
  char      c;

  while ( (c=*Key++) != '\0' )
  {
    Hash = (Hash<<5) - Hash + (TsmMSHash) c;
  }

  return 1 + (Hash % this->Size);
}

static TsmBool TsmMapString_LT( TsmMSHash Hash1, char* Key1, TsmMSHash Hash2, char* Key2 )
{
  return ( (Hash1<Hash2)
        || ( (Hash1==Hash2) && (strcmp(Key1,Key2)<0) ) );
}

static int TsmMapString_SeekOvw( TsmMapString* this, char* Key, TsmMSHash Hash )
{
  TsmMSOvwEntry*  pEntry;
  int             iLow = 0;
  int             iMid;
  int             iHigh = this->Overflow.Number-1;

  iMid = (iLow+iHigh)/2;
  while ( iLow != iMid )
  {
    pEntry = (TsmMSOvwEntry*) this->Overflow.Items[ iMid ];
    if ( TsmMapString_LT( Hash, Key, pEntry->Hash, pEntry->Key ) )
    {
      iHigh = iMid;
    }
    else
    {
      iLow = iMid;
    }
    iMid = (iLow+iHigh)/2;
  }

  return iLow;
}

static void TsmMapString_AddOvw( TsmMapString* this, char* Key, TsmMSHash Hash, TsmMapStringValue Value )
{
  int             iSeek;
  TsmMSOvwEntry*  pEntry;

  iSeek = TsmMapString_SeekOvw( this, Key, Hash );
  pEntry = (TsmMSOvwEntry*) Tsm_Malloc( sizeof(TsmMSOvwEntry) );
  pEntry->Hash  = Hash;
  pEntry->Key   = Key;
  pEntry->Value = Value;
  TsmArray_Insert( &this->Overflow, iSeek+1, pEntry );
}

void TsmMapString_Create( TsmMapString* this )
{
  TsmMSOvwEntry*  pEntry;

  this->Size = TSM_NOF_MAP_STRING;
  this->Items = Tsm_Malloc( ( this->Size+1 ) * sizeof(TsmMSEntry) );
  this->Fills = Tsm_Malloc( ( this->Size+1 ) * sizeof(unsigned char) );
  memset( this->Items, 0, ( this->Size+1 ) * sizeof(TsmMSEntry) );
  memset( this->Fills, 0, ( this->Size+1 ) * sizeof(unsigned char) );

  TsmArray_Malloc( &this->Overflow, 8, 8 );

  pEntry = Tsm_Malloc( sizeof(TsmMSOvwEntry) );
  pEntry->Hash  = 0;
  pEntry->Key   = "";
  pEntry->Value = NULL;
  TsmArray_Add( &this->Overflow, pEntry );

  pEntry = Tsm_Malloc( sizeof(TsmMSOvwEntry) );
  pEntry->Hash  = this->Size + 2;
  pEntry->Key   = NULL;
  pEntry->Value = NULL;
  TsmArray_Add( &this->Overflow, pEntry );
}

void TsmMapString_Free( TsmMapString* this )
{
  int             i;
  TsmMSOvwEntry*  pEntry;

  char            a[100];
  Tsm_Lgl( 2, "" );
  sprintf( a, "TsmMapString #Entries:   %d", this->Number );
  Tsm_Lgl( 2, a );
  sprintf( a, "TsmMapString #Overflows: %d", this->Overflow.Number-2 );
  Tsm_Lgl( 2, a );

  for ( i=0; i<this->Size; i++ )
  {
    if ( TsmMSFill_NOK != this->Fills[ i ] )
    {
      Tsm_Free( this->Items[ i ].Key );
      Tsm_Free( this->Items[ i ].Value );
    }
  }

  /*
   * Skip both sentinels.
   */
  for ( i=1; i<this->Overflow.Number-1; i++ )
  {
    pEntry = (TsmMSOvwEntry*) this->Overflow.Items[ i ];
    Tsm_Free( pEntry->Key );
    Tsm_Free( pEntry->Value );
  }
}

void TsmMapString_Delete( TsmMapString* this )
{
  TsmArray_FreeItems( &this->Overflow );
  TsmArray_Free( &this->Overflow );
  Tsm_Free( this->Items );
  Tsm_Free( this->Fills );
}

void TsmMapString_Add( TsmMapString* this, char* Key, TsmMapStringValue Value )
{
  TsmMSHash Hash = TsmMapString_Hash( this, Key );

  switch ( this->Fills[ Hash ] )
  {
  case TsmMSFill_NOK :
    this->Items[ Hash ].Key = Key;
    this->Items[ Hash ].Value = Value;
    this->Fills[ Hash ] = TsmMSFill_OK;
    break;
  case TsmMSFill_OK :
    TsmMapString_AddOvw( this, Key, Hash, Value );
    this->Fills[ Hash ] = TsmMSFill_OVW;
    break;
  case TsmMSFill_OVW :
    TsmMapString_AddOvw( this, Key, Hash, Value );
    break;
  }
  this->Number++;
}

TsmBool TsmMapString_Find( TsmMapString* this, char* Key, TsmMapStringValue* pValue )
{
  TsmBool         bFound = FALSE;
  TsmMSHash       Hash = TsmMapString_Hash( this, Key );
  TsmMSEntry*     pEntry;
  int             iSeek;
  TsmMSOvwEntry*  pOvwEntry;

  switch ( this->Fills[ Hash ] )
  {
  case TsmMSFill_NOK :
    break;
  case TsmMSFill_OK :
    pEntry = &this->Items[ Hash ];
    bFound = (strcmp(Key,pEntry->Key)==0);
    *pValue = pEntry->Value;
    break;
  case TsmMSFill_OVW :
    pEntry = &this->Items[ Hash ];
    if ( strcmp(Key,pEntry->Key)==0 )
    {
      bFound = TRUE;
      *pValue = pEntry->Value;
    }
    else
    {
      iSeek = TsmMapString_SeekOvw( this, Key, Hash );
      pOvwEntry = (TsmMSOvwEntry*) this->Overflow.Items[ iSeek ];
      bFound = ( (Hash==pOvwEntry->Hash) && (strcmp(Key,pOvwEntry->Key)==0) );
      *pValue = pOvwEntry->Value;
    }
    break;
  }
  return bFound;
}
