/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmFsm.c
 * version 1
 */
#include <string.h>
#include <stdio.h>
#include "TsmGen.h"
#include "TsmCompound.h"
#include "TsmFsm.h"
#include "TsmHandler.h"
#include "TsmHandlerAction.h"
#include "TsmRecorder.h"

TsmFsm* TsmFsm_Ctor( void )
{
  TsmFsm* this = (TsmFsm*) Tsm_Malloc( sizeof(TsmFsm) );
  TsmFsm_Ftor( this );
  return this;
}

void TsmFsm_Ftor( TsmFsm* this )
{
  TsmCompound_Ftor( (TsmCompound*) this );
  this->Class = "TsmFsm";
  this->mNrRegions = 0;
  this->mRegions = NULL;

  this->ExecuteTriggerFsm = TsmFsm_ExecuteTriggerFsm;
  this->ExecuteTriggerless = (Tsm_pfStHdlr) TsmFsm_ExecuteTriggerless;
  this->TsmReset1 = TsmFsm_TsmReset1;
  this->TsmReset2 = TsmFsm_TsmReset2;
  this->TsmSnap = (void (*)( TsmState*, TsmStr )) TsmFsm_TsmSnap;
}

void TsmFsm_ExecuteTriggerFsm( TsmFsm* this, TsmHandler* oHandler )
{
  this->ExecuteTrigger( (TsmState*) this, oHandler );
  if ( ! oHandler->bTaken )
  {
    char *s;
    
    s = Tsm_Malloc( 2*TSM_NCH_CLASSNAME + TSM_NCH_FULLNAME );
    sprintf( s, "**** UNHANDLED FUNCTION : %s @ %s : ",
      oHandler->Trigger->ClassName( (TsmObject*) oHandler->Trigger ),
      this->ClassName( (TsmObject*) this ) );
    this->FullName( (TsmBase*) this, s+strlen(s) );
    TsmRecorder_UnhandledFunction( s );
    Tsm_Free( s );
  }
}

void TsmFsm_ExecuteTriggerless( TsmFsm* this, TsmHandler* oHandler )
{
  oHandler->Action = TsmHandlerAction_QUIT;
}

void TsmFsm_TsmReset1( TsmFsm* this, TsmHandler* oHandler )
{
  this->AtExit( (TsmState*) this, oHandler );
}

void TsmFsm_TsmReset2( TsmFsm* this, TsmHandler* oHandler )
{
  this->Initial( (TsmState*) this, oHandler );
}

void TsmFsm_TsmSnap( TsmFsm* this, TsmStr sout )
{
  strcpy( sout, ":" );
  TsmCompound_TsmSnap( (TsmCompound*) this, sout+strlen(sout) );
}
