/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmRegion.c
 * version 1
 */
#include <string.h>
#include <stdio.h>
#include "TsmGen.h"
#include "TsmState.h"
#include "TsmRegion.h"
#include "TsmHandler.h"
#include "TsmHandlerAction.h"
#include "TsmProspect.h"
#include "TsmException.h"

TsmRegion* TsmRegion_Ctor( void )
{
  TsmRegion* this = (TsmRegion*) Tsm_Malloc( sizeof(TsmRegion) );
  TsmRegion_Ftor( this );
  return this;
}

void TsmRegion_Ftor( TsmRegion* this )
{
  TsmState_Ftor( (TsmState*) this );
  this->Class = "TsmRegion";
  this->mNrStates = 0;
  this->mStates = NULL;
  this->mCurrentState = NULL;
  this->mbFinal = FALSE;

  this->Gtor = (Tsm_pfObj) TsmRegion_Gtor;
  this->Initial = (Tsm_pfStHdlr) TsmRegion_Initial;
  this->ExecuteTrigger = (Tsm_pfStHdlr) TsmRegion_ExecuteTrigger;
  this->ExecuteTriggerless = (Tsm_pfStHdlr) TsmRegion_ExecuteTriggerless;
  this->AtExit = (Tsm_pfStHdlr) TsmRegion_AtExit;
  this->Reset = TsmRegion_Reset;
  this->TsmSnap = (void (*)( TsmState*, TsmStr )) TsmRegion_TsmSnap;
  this->TsmCheckVisit = (void (*)( TsmState* )) TsmRegion_TsmCheckVisit;
}

void TsmRegion_Gtor( TsmRegion* this )
{
  int i;

  for ( i=0; i<this->mNrStates; i++ )
  {
    this->mStates[i]->Dtor( (TsmObject*) this->mStates[i] );
  }
}

/**
 * This function should be overridden, or never be called.
 */
void TsmRegion_Initial( TsmRegion* this, TsmHandler* oHandler )
{
  char* sException;

  sException = Tsm_Malloc( TSM_NCH_FULLNAME );
  sprintf( sException, "9600: Region without Initial state @ " );
  this->FullName( (TsmBase*) this, sException+strlen(sException) );
  TsmException_Quit( sException );
  Tsm_Free( sException );
}

void TsmRegion_ExecuteTrigger( TsmRegion* this, TsmHandler* oHandler )
{
  int Prospect;

  if ( this->mbFinal )
  {
    oHandler->Action = TsmHandlerAction_FINAL;
    return;
  }

  Prospect = this->GetProspect( (TsmState*) this, oHandler->Trigger->PortId, oHandler->Trigger->Command );
  switch ( Prospect )
  {
  case TsmProspect_NOK :
    oHandler->Action = TsmHandlerAction_NOK;
    return;
  case TsmProspect_ANY :
    if ( (oHandler->Prospect & TsmProspect_OK) != 0 )
    { /*
       * Execute the trigger at the higher level.
       */
      oHandler->Action = TsmHandlerAction_NOK;
      return;
    }
    break;
  default :
    break;
  }

  this->mCurrentState->ExecuteTrigger( this->mCurrentState, oHandler );
  this->mCurrentState->ExecuteHandler( this->mCurrentState, oHandler );
  this->ExecuteHandler( (TsmState*) this, oHandler );
  this->mbFinal = ( oHandler->Action == TsmHandlerAction_FINAL );
}

void TsmRegion_ExecuteTriggerless( TsmRegion* this, TsmHandler* oHandler )
{ /*
   * Prevent redefining oHandler->Action
   */
}

void TsmRegion_AtExit( TsmRegion* this, TsmHandler* oHandler )
{
  if ( NULL != this->mCurrentState )
  {
    this->mCurrentState->AtExit( this->mCurrentState, oHandler );
    this->Reset( this );
  }
}

void TsmRegion_Reset( TsmRegion* this )
{
  this->mCurrentState = NULL;
  this->mbFinal = FALSE;
}

void TsmRegion_TsmSnap( TsmRegion* this, TsmStr sout )
{
  *sout = '\0';
  if ( NULL != this->mCurrentState )
  {
    this->mCurrentState->TsmSnap( this->mCurrentState, sout );
  }
}

void TsmRegion_TsmCheckVisit( TsmRegion* this )
{
  int i;

  for ( i=0; i<this->mNrStates; i++ )
  {
    this->mStates[i]->TsmCheckVisit( this->mStates[i] );
  }
}
