/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmSubFsm.c
 * version 1
 */
#include <string.h>
#include <stdio.h>
#include "TsmGen.h"
#include "TsmState.h"
#include "TsmSubFsm.h"
#include "TsmFsm.h"
#include "TsmHandler.h"
#include "TsmHandlerAction.h"

TsmSubFsm* TsmSubFsm_Ctor( void )
{
  TsmSubFsm* this = (TsmSubFsm*) Tsm_Malloc( sizeof(TsmSubFsm) );
  TsmSubFsm_Ftor( this );
  return this;
}

void TsmSubFsm_Ftor( TsmSubFsm* this )
{
  TsmState_Ftor( (TsmState*) this );
  this->Class = "TsmSubFsm";
  this->oFsm = NULL;

  this->Gtor = (Tsm_pfObj) TsmSubFsm_Gtor;
  this->Initial = (Tsm_pfStHdlr) TsmSubFsm_Initial;
  this->ExecuteTrigger = (Tsm_pfStHdlr) TsmSubFsm_ExecuteTrigger;
  this->ExecuteTriggerSubFsm = TsmSubFsm_ExecuteTriggerSubFsm;
  this->AtMyExit = TsmSubFsm_AtMyExit;
  this->AtExit = (Tsm_pfStHdlr) TsmSubFsm_AtExit;
  this->TsmSnap = (void (*)( TsmState*, TsmStr )) TsmSubFsm_TsmSnap;
  this->TsmCheckVisit = (void (*)( TsmState* )) TsmSubFsm_TsmCheckVisit;
  this->SwapFsms = TsmSubFsm_SwapFsms;
  this->IsTsmSubFsm = (TsmBool (*)( TsmBase* )) TsmSubFsm_IsTsmSubFsm;
}

void TsmSubFsm_Gtor( TsmSubFsm* this )
{
  this->oFsm->Gtor( (TsmObject*) this->oFsm );
}

void TsmSubFsm_Initial( TsmSubFsm* this, TsmHandler* oHandler )
{
  oHandler->Action = TsmHandlerAction_ENTER;
  this->oFsm->ExecuteHandler( (TsmState*) this->oFsm, oHandler );

  if ( oHandler->Action == TsmHandlerAction_FINAL )
  {
    this->ExecuteTriggerless( (TsmState*) this, oHandler );
  }

  if ( oHandler->Action == TsmHandlerAction_QUIT )
  {
    this->ExecuteTriggerless( (TsmState*) this, oHandler );
  }
}

void TsmSubFsm_ExecuteTrigger( TsmSubFsm* this, TsmHandler* oHandler )
{
  int ProspectSelf = this->GetProspect( (TsmState*) this, oHandler->Trigger->PortId, oHandler->Trigger->Command );
  oHandler->Prospect |= ProspectSelf;

  this->oFsm->ExecuteTrigger( (TsmState*) this->oFsm, oHandler );
  this->oFsm->ExecuteHandler( (TsmState*) this->oFsm, oHandler );

  if ( oHandler->Action == TsmHandlerAction_FINAL )
  {
    this->ExecuteTriggerless( (TsmState*) this, oHandler );
  }
  else
    if ( oHandler->Action == TsmHandlerAction_QUIT )
    {
      this->ExecuteTriggerless( (TsmState*) this, oHandler );
    }
    else
      if ( ! oHandler->bTaken )
      {
        /* Process the trigger itself, if transition available. */
        this->ExecuteTriggerSubFsm( this, oHandler );
      }
}

void TsmSubFsm_ExecuteTriggerSubFsm( TsmSubFsm* this, TsmHandler* oHandler )
{
  /* Override for all the SubFsm triggers. */
  oHandler->Action = TsmHandlerAction_NOK;
}

void TsmSubFsm_AtMyExit( TsmSubFsm* this, TsmHandler* oHandler )
{
}

void TsmSubFsm_AtExit( TsmSubFsm* this, TsmHandler* oHandler )
{
  this->oFsm->AtExit( (TsmState*) this->oFsm, oHandler );
  this->AtMyExit( this, oHandler );
}

void TsmSubFsm_TsmSnap( TsmSubFsm* this, TsmStr sout )
{
  sprintf( sout, "%s+", this->TsmName );
  this->oFsm->TsmSnap( (TsmState*) this->oFsm, sout+strlen(sout) );
}

void TsmSubFsm_TsmCheckVisit( TsmSubFsm* this )
{
  this->oFsm->TsmCheckVisit( (TsmState*) this->oFsm );
}

TsmFsm* TsmSubFsm_SwapFsms( TsmSubFsm* this, TsmFsm* Target2Source )
{
  TsmFsm* h = this->oFsm;
  TsmBase* hp = this->oFsm->TsmParent;

  this->oFsm = Target2Source;
  h->TsmParent = Target2Source->TsmParent;
  this->oFsm->TsmParent = hp;

  return h;
}

TsmBool TsmSubFsm_IsTsmSubFsm( TsmSubFsm* this )
{
  return TRUE;
}
