/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmQueue.c
 * version 1
 */
#include <string.h>
#include <stdio.h>
#include "TsmGen.h"
#include "TsmObject.h"
#include "TsmQueue.h"
#include "TsmMutex.h"
#include "TsmFifo_Envelope.h"
#include "TsmEnvelope.h"
#include "TsmSupervisor.h"

TsmQueue* TsmQueue_Ctor( void )
{
  TsmQueue* this = (TsmQueue*) Tsm_Malloc( sizeof(TsmQueue) );
  TsmQueue_Ftor( this );
  return this;
}

void TsmQueue_Ftor( TsmQueue* this )
{
  TsmObject_Ftor( (TsmObject*) this );
  this->Class = "TsmQueue";
  this->mMaxQueueSize = 0;
  this->mSeqId = 0;

  this->Gtor = (Tsm_pfObj) TsmQueue_Gtor;
  this->GetMaxQueueSize = TsmQueue_GetMaxQueueSize;
  this->GetSeqId = TsmQueue_GetSeqId;
  this->SetSeqId = TsmQueue_SetSeqId;
  this->Put = TsmQueue_Put;
  this->Dequeue = TsmQueue_Dequeue;
  this->Empty = TsmQueue_Empty;
  this->Number = TsmQueue_Number;
  this->Snap = TsmQueue_Snap;

  TsmMutex_Ftor( &this->mLockMutex );
  TsmFifo_Envelope_Initialise( &this->mFifo, TSM_NOF_QUEUE_ENVELOPES, this->mEnvelopes );
}

void TsmQueue_Gtor( TsmQueue* this )
{
  TsmMutex_Gtor( &this->mLockMutex );
}

int TsmQueue_GetMaxQueueSize( TsmQueue* this )
{
  return this->mMaxQueueSize;
}

int TsmQueue_GetSeqId( TsmQueue* this )
{
  return this->mSeqId;
}

void TsmQueue_SetSeqId( TsmQueue* this, int SeqId )
{
  this->mSeqId = SeqId;
}

void TsmQueue_Put( TsmQueue* this, TsmEnvelope* oEnvelope )
{
  TsmMutex_WaitOne( &this->mLockMutex );

  if ( TsmSupervisor_QueueOverflow( this->mFifo.Number ) )
  {
    oEnvelope->mTrigger->DBU( oEnvelope->mTrigger );
  }
  else
  {
    TsmFifo_Envelope_Put( &this->mFifo, oEnvelope );
    if ( this->mMaxQueueSize < this->mFifo.Number )
    {
      this->mMaxQueueSize = this->mFifo.Number;
    }
  }
  TsmMutex_Release( &this->mLockMutex );
}

TsmBool TsmQueue_Dequeue( TsmQueue* this, TsmEnvelope* oEnvelope )
{
  TsmBool bDequeue;

  TsmMutex_WaitOne( &this->mLockMutex );
  bDequeue = TsmFifo_Envelope_Get( &this->mFifo, oEnvelope );
  TsmMutex_Release( &this->mLockMutex );

  return bDequeue;
}

void TsmQueue_Empty( TsmQueue* this )
{
  TsmEnvelope lEnvelope;

  while ( TsmQueue_Dequeue( this, &lEnvelope ) )
  {
    lEnvelope.mTrigger->Dtor( (TsmObject*) lEnvelope.mTrigger );
  }
}

int TsmQueue_Number( TsmQueue* this )
{
  return this->mFifo.Number;
}

void TsmQueue_Snap( TsmQueue* this, TsmStr sout )
{
  int           i;
  int           j;
  TsmEnvelope*  pEnvelope;

  TsmMutex_WaitOne( &this->mLockMutex );

  j = this->mFifo.Head;
  for ( i=0; i<this->mFifo.Number; i++ )
  {
    pEnvelope = &this->mFifo.pItems[j];
    TsmEnvelope_Snap( pEnvelope, sout+strlen(sout) );
    strcat( sout, "," );
    if ( ++j >= this->mFifo.Size )
    {
      j = 0;
    }
  }
  TsmMutex_Release( &this->mLockMutex );
}
