/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmSvrChecker.c
 * version 4
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "OSAL_Gen.h"
#include "OSAL_Thread.h"
#include "TsmSvrChecker.h"
#include "Tsm.h"
#include "TsmGen.h"
#include "TsmSvrHelpers.h"
#include "TsmArray.h"
#include "TsmException.h"
#include "TsmSupervisor.h"
#include "TsmRecorder.h"

#define TsmSvrChecker_BUFLEN  8192

void    TsmSvrChecker_DeleteScenItems( TsmArray* oScenItems );
TsmBool TsmSvrChecker_AllThreadsWaiting( TsmSvrChecker* this );
TsmBool TsmSvrChecker_AllThreadsSleeping( TsmSvrChecker* this );
void    TsmSvrChecker_CheckAllThreadsWaiting( TsmSvrChecker* this );
void    TsmSvrChecker_Reset( TsmSvrChecker* this );
void    TsmSvrChecker_Free( TsmSvrChecker* this );
void    TsmSvrChecker_Pause( TsmThreadData* oThreadData );
void    TsmSvrChecker_Continue( TsmThreadData* oThreadData );
void    TsmSvrChecker_ReplayNative( TsmSvrChecker* this, TsmScenItem* oItem );
void    TsmSvrChecker_ReplayScenItem( TsmSvrChecker* this, TsmScenItem* Item );
void    TsmSvrChecker_Replay( TsmSvrChecker* this );
TsmScenItem* TsmSvrChecker_GetThreadDataRV2( TsmSvrChecker* this );
void    TsmSvrChecker_ReplayExtend( TsmSvrChecker* this, TsmScenario* oScenario );
void    TsmSvrChecker_AddToCurrentScenario( TsmSvrChecker* this, TsmArray* oScenItems );
void    TsmSvrChecker_AddScenarioNatives( TsmSvrChecker* this, TsmArray* oScenItems, TsmBool bClient );
void    TsmSvrChecker_AddScenarioCbThread( TsmSvrChecker* this, TsmArray* oScenItems );
void    TsmSvrChecker_AddScenarioThreads( TsmSvrChecker* this, TsmArray* oScenItems );
int     TsmSvrChecker_GetRandomInt( int iMax );
void    TsmSvrChecker_GetDuration( long duration, TsmStr sout );
void    TsmSvrChecker_TxtInt( char* s, int k, TsmStr sout );
void    TsmSvrChecker_TxtIntLog( char* s, int k );

static int            TsmSvrChecker_mThreadId = 0;
static TsmSvrChecker* TsmSvrChecker_SingleInstance = NULL;

TsmSvrChecker* TsmSvrChecker_Instance( void )
{
  if ( NULL == TsmSvrChecker_SingleInstance )
  {
    TsmSvrChecker_SingleInstance = TsmSvrChecker_Ctor();
  }
  return TsmSvrChecker_SingleInstance;
}

void TsmSvrChecker_Cleanup( void )
{
  if ( NULL != TsmSvrChecker_SingleInstance )
  {
    TsmSvrChecker_Dtor( TsmSvrChecker_SingleInstance );
    TsmSvrChecker_SingleInstance = NULL;
  }
}

TsmSvrChecker* TsmSvrChecker_Ctor( void )
{
  TsmSvrChecker* this = (TsmSvrChecker*) Tsm_Malloc( sizeof(TsmSvrChecker) );
  TsmSvrChecker_Ftor( this );
  return this;
}

void TsmSvrChecker_Ftor( TsmSvrChecker* this )
{
  memset( this, 0, sizeof(TsmSvrChecker) );
  TsmTriggerSem_Ftor( &this->mTriggerSem );
  TsmMutex_Ftor( &this->LockNatives );
  TsmArray_Malloc( &this->Classes, 8, 8 );
}

void TsmSvrChecker_Gtor( TsmSvrChecker* this )
{
  TsmArray_Free( &this->Classes );
  TsmMutex_Gtor( &this->LockNatives );
  TsmTriggerSem_Gtor( &this->mTriggerSem );
}

void TsmSvrChecker_Dtor( TsmSvrChecker* this )
{
  TsmSvrChecker_Gtor( this );
  Tsm_Free( this );
}

TsmArray* TsmSvrChecker_GetClasses( TsmSvrChecker* this )
{
  return &this->Classes;
}

int TsmSvrChecker_Register( TsmSvrChecker* this, TsmThread* oTsmThread, TsmThreadType eType, OSAL_Handle hThreadBid )
{
  TsmThreadData* lThreadData = NULL;

  switch ( eType )
  {
  case TsmThreadType_SUPERVISOR:
    lThreadData = &this->MyThreadData;
    break;
  case TsmThreadType_CB_THREAD:
    lThreadData = &this->CbThreadData;
    break;
  case TsmThreadType_THREAD:
    lThreadData = &this->mThreadDatas[ hThreadBid-1 ];
    break;
  }
 
  if ( lThreadData->TsmThread != NULL )
  {
    TsmException_Quit( "TismBUG: Doubly registered thread" );
  }

  lThreadData->Type      = eType;
  lThreadData->ThreadId  = TsmSvrChecker_mThreadId++;
  lThreadData->TsmThread = oTsmThread;
  lThreadData->State     = TsmThreadState_RUNNING;

  return lThreadData->ThreadId;
}

static TsmThreadData* TsmSvrChecker_GetData( TsmSvrChecker* this, OSAL_Handle ThreadBid )
{
  TsmThreadData* lThreadData;

  if ( ThreadBid == this->CbThreadData.TsmThread->GetThread(this->CbThreadData.TsmThread) )
  {
    lThreadData = &this->CbThreadData;
  }
  else
  {
    lThreadData = &this->mThreadDatas[ ThreadBid-1 ];
    if ( lThreadData->TsmThread == NULL )
    {
      TsmException_Quit( "TismBUG: Unregistered thread" );
    }
  }
  return lThreadData;
}

static void TsmSvrChecker_SetStateData( TsmSvrChecker* this, TsmThreadData* lThreadData, TsmThreadState eState )
{
  lThreadData->State = eState;
  switch ( lThreadData->Type )
  {
  case TsmThreadType_SUPERVISOR:
    switch ( lThreadData->State )
    {
    case TsmThreadState_RUNNING:
    case TsmThreadState_INTERRUPTED:
      TsmException_Quit( "TismBUG: Supervisor should not report" );
      break;
    case TsmThreadState_WAITING_MSG:
    case TsmThreadState_WAITING_RV:
    case TsmThreadState_WAITING_RV2:
      TsmException_Quit( "TismBUG: Supervisor should not wait" );
      break;
    }
    break;
  case TsmThreadType_CB_THREAD:
  case TsmThreadType_THREAD:
    switch ( lThreadData->State )
    {
    case TsmThreadState_RUNNING:
      break;
    case TsmThreadState_WAITING_MSG:
      TsmTriggerSem_Release( &this->mTriggerSem );
      TsmThread_TriggerWait( lThreadData->TsmThread );
      break;
    case TsmThreadState_WAITING_RV:
      TsmTriggerSem_Release( &this->mTriggerSem );
      break;
    case TsmThreadState_WAITING_RV2:
      if ( lThreadData->bInterrupt )
      {
        lThreadData->bInterrupt = FALSE;
        TsmException_Rv();
      }
      TsmTriggerSem_Release( &this->mTriggerSem );
      TsmThread_TriggerWait( lThreadData->TsmThread );
      break;
    case TsmThreadState_INTERRUPTED:
      lThreadData->State = TsmThreadState_RUNNING;
      break;
    }
    break;
  }
}

void TsmSvrChecker_SetState( TsmSvrChecker* this, OSAL_Handle ThreadBid, TsmThreadState eState )
{
  TsmThreadData* lThreadData = TsmSvrChecker_GetData( this, ThreadBid );

  TsmSvrChecker_SetStateData( this, lThreadData, eState );
}

void TsmSvrChecker_SetStateRv( TsmSvrChecker* this, OSAL_Handle ThreadBid, TsmThread* oThread )
{
  TsmThreadData* lThreadData = TsmSvrChecker_GetData( this, ThreadBid );

  lThreadData->RvThread = oThread;
  TsmSvrChecker_SetStateData( this, lThreadData, TsmThreadState_WAITING_RV );
}

void TsmSvrChecker_NativesReset( TsmSvrChecker* this )
{
  int i;

  for( i=0; i<this->Classes.Number; i++ )
  {
    TsmClass* cl = (TsmClass*) TsmArray_At( &this->Classes, i );
    TsmNativeLists_Clear( &cl->Natives );
  }
}

void TsmSvrChecker_NativeEntry( TsmSvrChecker* this, TsmClass* oOwner, const int aNatives[], int NrNatives )
{
  TsmNativeLists* lLists;
  int             i;

  TsmMutex_WaitOne( &this->LockNatives );
  lLists = &oOwner->Natives;
  for ( i=0; i<NrNatives; i++ )
  {
    int n = aNatives[i];
    if ( n < 0 )
    {
      TsmList_Add( &lLists->ClientNatives, (TsmListValue) n );
    }
    else
    {
      TsmList_Add( &lLists->ServerNatives, (TsmListValue) n );
    }
  }
  TsmMutex_Release( &this->LockNatives );
}

void TsmSvrChecker_NativeExit( TsmSvrChecker* this, TsmClass* oOwner, const int aNatives[], int NrNatives )
{
  TsmNativeLists* lLists;
  int             i;

  TsmMutex_WaitOne( &this->LockNatives );
  lLists = &oOwner->Natives;
  for ( i=0; i<NrNatives; i++ )
  {
    int n = aNatives[i];
    if ( n < 0 )
    {
      TsmList_Remove( &lLists->ClientNatives, (TsmListValue) n );
    }
    else
    {
      TsmList_Remove( &lLists->ServerNatives, (TsmListValue) n );
    }
  }
  TsmMutex_Release( &this->LockNatives );
}

void TsmSvrChecker_AddQueueOverflow( TsmSvrChecker* this )
{
  TsmScenItem* lScenItem = TsmScenItem_Ctor();

  lScenItem->Kind = TsmScenItemKind_QO;
  TsmScenario_Add( this->CurrentScenario, lScenItem );
  this->CurrentScenItemIndex++;
  this->bQueueOverflow = TRUE;
}

TsmBool TsmSvrChecker_ChoiceOKVerify( TsmSvrChecker* this, int MaxNrChoices, int ChoiceIndex, char* sGuard )
{
  TsmScenItem* lScenItem;

  if ( this->CurrentScenItemIndex < this->CurrentScenario->Items.Number )
  { /*
     * Still replaying.
     */
    lScenItem = (TsmScenItem*) TsmArray_At( &this->CurrentScenario->Items, this->CurrentScenItemIndex );
    TsmAssert( lScenItem->Kind == TsmScenItemKind_CHOICE );
    if ( lScenItem->Index != ChoiceIndex )
    {
      return FALSE;
      /***********/
    }

    this->CurrentScenItemIndex++;
    if ( this->CurrentScenItemIndex == this->CurrentScenario->Items.Number )
    { /*
       * 'Execution' of the last item is happening now.
       */
      TsmRecorder_Enable();
    }
  }
  else
  { /*
     * The scenario has been replayed fully, and
     * now encounters a Choice node.
     */
    TsmScenario*  lNewScenario;
    int           i;

    TsmAssert( ChoiceIndex == 1 );
    for ( i=2; i<=MaxNrChoices; i++ )
    {
      lScenItem = TsmScenItem_Ctor();
      lScenItem->Kind = TsmScenItemKind_CHOICE;
      lScenItem->Index = i;
      lNewScenario = TsmScenario_ExtendChoice( this->CurrentScenario, lScenItem );
      TsmStackList_Push( &this->FutureScenarios, (TsmStackEntry *) lNewScenario );
    }

    lScenItem = TsmScenItem_Ctor();
    lScenItem->Kind = TsmScenItemKind_CHOICE;
    lScenItem->Index = 1;
    TsmScenario_Add( this->CurrentScenario, lScenItem );
    this->CurrentScenItemIndex++;
  }
  Tsm_Seq0( "H," );
  Tsm_Seq( sGuard );
  return TRUE;
}

void TsmSvrChecker_Verify( TsmSvrChecker* this  )
{
  TsmArray              lScenItems;
  TsmBool               bProxyProtocol = FALSE;
  TsmBool               bClient;
  TsmSnapshot*          lSnapshot;
  TsmSnapdata*          lSnapdata;
  TsmBool               mbCurrent = FALSE;
  TsmStr                sb1;
  TsmStr                sb2;
  TsmStr                sb3;
  TsmStr                ss;
  char                  sl[ 32 ];
  TsmStr                sb;

  this->StartTime = time( NULL );

  sb = Tsm_Malloc( TsmSvrChecker_BUFLEN );
  TsmRecorder_Initialise();
  TsmMapString_Create( &this->Snapshots );
  TsmStackList_Initialise( &this->PastScenarios );
  TsmStackList_Initialise( &this->FutureScenarios );
  TsmArray_Malloc( &lScenItems, 8, 8 );

  if ( this->Classes.Number == 2 )
  {
    TsmClass* c0 = (TsmClass*) TsmArray_At( &this->Classes, 0 );
    TsmClass* c1 = (TsmClass*) TsmArray_At( &this->Classes, 1 );
    if ( ( c0->IsTsmProxy(c0) && c1->IsTsmProxyUser(c1) )
      || ( c1->IsTsmProxy(c1) && c0->IsTsmProxyUser(c0) ) )
    {
      bProxyProtocol = TRUE;
    }
  }
  this->CurrentScenario = TsmScenario_Ctor();
  TsmStackList_Push( &this->FutureScenarios, (TsmStackEntry *) this->CurrentScenario );

  mbCurrent = FALSE;
  while ( ! TsmStackList_Empty( &this->FutureScenarios ) )
  {
    this->CurrentScenario = (TsmScenario*) TsmStackList_Pop( &this->FutureScenarios );
    TsmStackList_Push( &this->PastScenarios, (TsmStackEntry *) this->CurrentScenario );

    if ( mbCurrent )
    {
      int  ParentId;

      mbCurrent = FALSE;

      ParentId = TsmScenario_GetParentId( this->CurrentScenario );
      sprintf( sb, "S,%d,%d", this->CurrentScenario->Id, ParentId );
      Tsm_Seq( sb );

      sprintf( sb, "Scenario [ %d  :  %d ] -------------------------------------------------------",
        this->CurrentScenario->Id, ParentId );
      Tsm_Lgl( 2, sb );
      TsmScenario_ToString( this->CurrentScenario, sb );
      Tsm_Lgl( 3, sb );

      if ( this->CurrentScenario->Items.Number > 0 )
      {
        TsmScenItem* Si;

        TsmSvrChecker_CheckAllThreadsWaiting( this );
        Si = (TsmScenItem*) TsmArray_At( &this->CurrentScenario->Items, this->CurrentScenItemIndex++ );
        TsmSvrChecker_ReplayScenItem( this, Si );
        TsmAssert( this->CurrentScenItemIndex == this->CurrentScenario->Items.Number );
      }
      TsmSvrChecker_CheckAllThreadsWaiting( this );
    }
    else
    {
      int  ParentId;

      Tsm_Seq( "N" );
      TsmSvrChecker_Reset( this );

      ParentId = TsmScenario_GetParentId( this->CurrentScenario );
      sprintf( sb, "S,%d,%d", this->CurrentScenario->Id, ParentId );
      Tsm_Seq( sb );

      sprintf( sb, "Scenario [ %d  :  %d ] -------------------------------------------------------",
        this->CurrentScenario->Id, ParentId );
      Tsm_Lgl( 2, sb );
      TsmScenario_ToString( this->CurrentScenario, sb );
      Tsm_Lgl( 3, sb );

      TsmSvrChecker_Replay( this );
    }

    TsmSvrChecker_ReplayExtend( this, this->CurrentScenario );
    /*
     * Now, no threads have state RV2 !
     */
    sb1 = Tsm_Malloc( TsmSvrChecker_BUFLEN );
    TsmSvrChecker_SnappedClasses( this, sb1 );
    sb1 = Tsm_Restring( sb1 );

    sb2 = Tsm_Malloc( TsmSvrChecker_BUFLEN );
    this->CbThreadData.TsmThread->Snap( this->CbThreadData.TsmThread, sb2 );
    sb2 = Tsm_Restring( sb2 );

    sb3 = Tsm_Malloc( TsmSvrChecker_BUFLEN );
    TsmSvrChecker_SnappedThreads( this, sb3 );
    sb3 = Tsm_Restring( sb3 );

    lSnapshot = TsmSnapshot_Ctor( sb1, sb2, sb3 );
    TsmScenario_Update( this->CurrentScenario, lSnapshot );

    if ( this->bQueueOverflow )
    {
      this->bQueueOverflow = FALSE;
      TsmRecorder_QueueOverflow();
      continue;
      /*******/
    }

    Tsm_Seq0( "T," );
    Tsm_Seq( lSnapshot->SnapClasses );
    ss = Tsm_Malloc( TsmSvrChecker_BUFLEN );
    TsmSnapshot_ToString( lSnapshot, ss );

    if ( TsmMapString_Find( &this->Snapshots, ss, (void**) &lSnapdata ) )
    {
      Tsm_Free( ss );

      TsmSvrChecker_TxtInt( "Equal: ", lSnapdata->Scenario->Id, sl );
      Tsm_Lgl( 5, sl );
      Tsm_Seq2( "Q,", lSnapdata->Scenario->Id );
      if ( TsmScenario_HasCycle( this->CurrentScenario ) )
      {
        this->NumberOfCycles++;
        Tsm_Lgl( 4, "Cycle: -------------------------------------------------------" );
        Tsm_Seq( "Y" );
        if ( lSnapdata->NrCalls == TsmRecorder_GetNumberOfExtCalls() )
        { /*
           * The number of external calls retrieved from the previous
           * position in lSnapdata is equal to the current number of
           * external calls in the Recorder.
           */
          this->NumberOfLivelocks++;
          TsmRecorder_Show2( "**** LIVELOCK ", lSnapdata->NrRecords );
        }
      }
    }
    else
    {
      ss = Tsm_Restring( ss );

      Tsm_Lgl( 5, "Rply: -------------------------------------------------------" );
      Tsm_Lgl0( 5, "Snap:" );
      Tsm_Lgl( 5, ss );
      lSnapdata = TsmSnapdata_Ctor(
        TsmRecorder_GetNumberOfRecords(),
        TsmRecorder_GetNumberOfExtCalls(),
        this->CurrentScenario );
      TsmMapString_Add( &this->Snapshots, ss, lSnapdata );

      bClient = (this->CbThreadData.State == TsmThreadState_WAITING_MSG);
      TsmArray_Clear( &lScenItems );
      TsmSvrChecker_AddScenarioCbThread( this, &lScenItems );
      if ( bProxyProtocol )
      { /*
         * In case of a proxy protocol, TsmScenItemKind_NATIVE triggers are only
         * allowed in the absense of pending callbacks.
         */
        if ( lScenItems.Number == 0 )
        {
          TsmSvrChecker_AddScenarioNatives( this, &lScenItems, bClient );
          TsmSvrChecker_AddScenarioThreads( this, &lScenItems );
        }
      }
      else
      {
        TsmSvrChecker_AddScenarioNatives( this, &lScenItems, bClient );
        TsmSvrChecker_AddScenarioThreads( this, &lScenItems );
      }

      if ( lScenItems.Number == 0 )
      {
        this->NumberOfDeadlocks++;
        TsmRecorder_Show( "**** DEADLOCK " );
      }
      else
      {
        TsmSvrChecker_AddToCurrentScenario( this, &lScenItems );
        mbCurrent = TRUE;
      }
    }
  }

  /*
   * Delete all Snapdata.
   */
  TsmMapString_Free( &this->Snapshots );
  TsmMapString_Delete( &this->Snapshots );

  /*
   * Delete all this->Snapshots and Scenarios.
   */
  this->NumberOfScenarios = this->PastScenarios.Number;
  while ( ! TsmStackList_Empty( &this->PastScenarios ) )
  {
    TsmScenario* lTsmScenario = (TsmScenario*) TsmStackList_Pop( &this->PastScenarios );
    TsmScenario_Dtor( lTsmScenario );
  }

  /*
   * Cleanup.
   */
  TsmArray_Free( &lScenItems );
  TsmSvrChecker_Reset( this );
  TsmSvrChecker_Free( this);
  TsmRecorder_Reset();
  TsmRecorder_Terminate();
  Tsm_Free( sb );
  this->StopTime = time( NULL );
}


TsmBool TsmSvrChecker_ChoiceOKSimulate( TsmSvrChecker* this, int MaxNrChoices, int ChoiceIndex )
{
  int v = MaxNrChoices + 1 - ChoiceIndex;

  return (TsmSvrChecker_GetRandomInt( v ) == 0);
}

void TsmSvrChecker_Simulate( TsmSvrChecker* this )
{
  TsmArray              lScenItems;
  TsmScenItem*          lScenItem;
  TsmBool               bProxyProtocol = FALSE;
  TsmBool               bClient;
  char                  ss[ 200 ];
  TsmBool               bCurrent = FALSE;

  srand( TsmSupervisor_mRandomSeed );
  this->StartTime = time( NULL );

  TsmRecorder_Initialise();
  TsmArray_Malloc( &lScenItems, 8, 8 );

  if ( this->Classes.Number == 2 )
  {
    TsmClass* c0 = (TsmClass*) TsmArray_At( &this->Classes, 0 );
    TsmClass* c1 = (TsmClass*) TsmArray_At( &this->Classes, 1 );
    if ( ( c0->IsTsmProxy(c0) && c1->IsTsmProxyUser(c1) )
      || ( c1->IsTsmProxy(c1) && c0->IsTsmProxyUser(c0) ) )
    {
      bProxyProtocol = TRUE;
    }
  }

  this->CurrentScenario = TsmScenario_Ctor();

  bCurrent = FALSE;
  this->NumberOfLoops = 0;
  do
  {
    if ( ++this->NumberOfLoops == 0 )
    {
      this->NumberOfLoops = 1;
    }

    sprintf( ss, "Loop: %ld", this->NumberOfLoops );
    Tsm_Lgl( 3, ss );
    if ( bCurrent )
    {
      int r;

      bCurrent = FALSE;

      TsmSvrChecker_CheckAllThreadsWaiting( this );
      r = TsmSvrChecker_GetRandomInt( lScenItems.Number );
      lScenItem = (TsmScenItem*) TsmArray_At( &lScenItems, r );
      TsmSvrChecker_ReplayScenItem( this, lScenItem );
      TsmSvrChecker_CheckAllThreadsWaiting( this );
    }
    else
    {
      TsmSvrChecker_Reset( this );
      TsmSvrChecker_Replay( this );
    }

    while ( NULL != ( lScenItem = TsmSvrChecker_GetThreadDataRV2( this ) ) )
    {
      TsmSvrChecker_ReplayScenItem( this, lScenItem );
      TsmScenItem_Dtor( lScenItem );
      TsmSvrChecker_CheckAllThreadsWaiting( this );
    }

    bClient = ( this->CbThreadData.State == TsmThreadState_WAITING_MSG );
    TsmSvrChecker_DeleteScenItems( &lScenItems );
    TsmSvrChecker_AddScenarioCbThread( this, &lScenItems );
    if ( bProxyProtocol )
    { /*
       * In case of a proxy protocol, TsmScenItemKind_NATIVE triggers are only
       * allowed in the absense of pending callbacks.
       */
      if ( lScenItems.Number == 0 )
      {
        TsmSvrChecker_AddScenarioNatives( this, &lScenItems, bClient );
        TsmSvrChecker_AddScenarioThreads( this, &lScenItems );
      }
    }
    else
    {
      TsmSvrChecker_AddScenarioNatives( this, &lScenItems, bClient );
      TsmSvrChecker_AddScenarioThreads( this, &lScenItems );
    }

    if ( lScenItems.Number == 0 )
    {
      this->NumberOfDeadlocks++;
      TsmRecorder_Show( "**** DEADLOCK " );
    }
    else
    {
      bCurrent = TRUE;
    }
  } while ( this->NumberOfLoops != TsmSupervisor_mMaxNrSimLoops );

  TsmSvrChecker_DeleteScenItems( &lScenItems );
  TsmArray_Free( &lScenItems );
  TsmScenario_Dtor( this->CurrentScenario );
  TsmSvrChecker_Reset( this );
  TsmSvrChecker_Free( this );
  TsmRecorder_Reset();
  TsmRecorder_Terminate();
  this->StopTime = time( NULL );
}

void TsmSvrChecker_Summary( TsmSvrChecker* this, TsmBool bVerify )
{
  int     i;
  char    st[ 32 ];
  double  DTime = difftime( this->StopTime, this->StartTime );
  long    duration = (long) (DTime);

  Tsm_Log( "" );
  Tsm_Log( "SUMMARY" );
  Tsm_Log( "-------" );
  if ( bVerify )
  {
    TsmSvrChecker_TxtIntLog( "Number of scenarios: ", this->NumberOfScenarios );
    TsmSvrChecker_TxtIntLog( "Number of cycles   : ", this->NumberOfCycles );
  }
  else
  {
    TsmSvrChecker_TxtIntLog( "Number of loops    : ", this->NumberOfLoops );
  }
  TsmSvrChecker_TxtIntLog( "Number of deadlocks: ", this->NumberOfDeadlocks );
  if ( bVerify )
  {
    TsmSvrChecker_TxtIntLog( "Number of livelocks: ", this->NumberOfLivelocks );
  }
  TsmSvrChecker_TxtIntLog( "Number of unhandled functions  : ", TsmRecorder_NumberOfUnhandledFunctions );
  TsmSvrChecker_TxtIntLog( "Number of unvisited states     : ", TsmRecorder_NumberOfUnvisitedStates );
  TsmSvrChecker_TxtIntLog( "Number of unvisited transitions: ", TsmRecorder_NumberOfUnvisitedTransitions );
  TsmSvrChecker_TxtIntLog( "Number of rendez-vous errors   : ", TsmRecorder_NumberOfSyncReturnErrors );
  TsmSvrChecker_TxtIntLog( "Number of queue overflows      : ", TsmRecorder_NumberOfQueueOverflows );
  sprintf( st, "Duration: " );
  TsmSvrChecker_GetDuration( duration, st+strlen(st) );
  Tsm_Log( st );
  Tsm_Log( "-------" );
  Tsm_Log( "Ext  Int  Thread's maximum queue sizes" );
  TsmThread_MaxQueueSizes( this->CbThreadData.TsmThread, st );
  Tsm_Log( st );

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      TsmThread_MaxQueueSizes( lTsmThread, st );
      Tsm_Log( st );
    }
  }
  Tsm_Log( "-------" );
}

void TsmSvrChecker_SnappedClasses( TsmSvrChecker* this, TsmStr sout )
{
  int i;
  int c = 0;

  *sout = '\0';
  for( i=0; i<this->Classes.Number; i++ )
  {
    TsmClass* cl = (TsmClass*) TsmArray_At( &this->Classes, i );
    sprintf( sout+strlen(sout), "%d", ++c );
    cl->TsmSnap( cl, sout+strlen(sout) );
  }
}

void TsmSvrChecker_SnappedThreads( TsmSvrChecker* this, TsmStr sout )
{
  int i;
  int t = 0;

  *sout = '\0';
  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      sprintf( sout+strlen(sout), " T_%d: ", ++t );
      lTsmThread->Snap( lTsmThread, sout+strlen(sout) );
    }
  }
}

void TsmSvrChecker_GetCurrentSituation( TsmSvrChecker* this, TsmStr sout )
{
  int i;
  int c = 0;
  TsmScenario* lScenario;

  sprintf( sout, "Classes:\n" );
  for( i=0; i<this->Classes.Number; i++ )
  {
    TsmClass* cl = (TsmClass*) TsmArray_At( &this->Classes, i );
    sprintf( sout+strlen(sout), "%3d ", ++c );
    cl->TsmSnap( cl, sout+strlen(sout) );
    strcat( sout, "\n" );
  }

  strcat( sout, "Scenario:\n" );
  lScenario = (TsmScenario*) TsmStackList_Peek( &this->PastScenarios );
  TsmScenario_ToString( lScenario, sout+strlen(sout) );
}

void TsmSvrChecker_GetThreadStates( TsmSvrChecker* this, char* sHeader, TsmStr sout )
{
  int i;

  sprintf( sout, "%s\n", sHeader );
  TsmThreadData_ToState( &this->MyThreadData, sout+strlen(sout) );
  strcat( sout, "\n" );
  TsmThreadData_ToState( &this->CbThreadData, sout+strlen(sout) );
  strcat( sout, "\n" );

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( NULL != this->mThreadDatas[i].TsmThread )
    {
      TsmThreadData_ToState( &this->mThreadDatas[i], sout+strlen(sout) );
      strcat( sout, "\n" );
    }
  }
}

void TsmSvrChecker_DeleteScenItems( TsmArray* oScenItems )
{
  TsmArray_DtorItems( oScenItems );
}

TsmBool TsmSvrChecker_AllThreadsWaiting( TsmSvrChecker* this )
{
  int i;

  if ( TsmThreadState_RUNNING == this->CbThreadData.State )
  {
    return FALSE;
  }

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( NULL != this->mThreadDatas[i].TsmThread )
    {
      if ( TsmThreadState_RUNNING == this->mThreadDatas[i].State )
      {
        return FALSE;
      }
    }
  }

  return TRUE;
}

TsmBool TsmSvrChecker_AllThreadsSleeping( TsmSvrChecker* this )
{
  return TRUE;
}

void TsmSvrChecker_CheckAllThreadsWaiting( TsmSvrChecker* this )
{
  for ( ; ; )
  {
    OSAL_ThreadYield();
    if ( TsmSvrChecker_AllThreadsWaiting( this ) )
    {
      break;
    }
    TsmTriggerSem_WaitOne( &this->mTriggerSem );
  }
}

void TsmSvrChecker_Reset( TsmSvrChecker* this )
{
  int  i;
  char st[ 32 ];

  this->CbThreadData.TsmThread->TsmReset1( this->CbThreadData.TsmThread );

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      lTsmThread->TsmReset1( lTsmThread );
    }
  }

  if ( TsmThreadState_WAITING_RV == this->CbThreadData.State )
  {
    Tsm_Lgl( 7, "Interrupt Cb" );
    this->CbThreadData.State = TsmThreadState_RUNNING;
    this->CbThreadData.bInterrupt = TRUE;
    this->CbThreadData.RvThread->Interrupt( this->CbThreadData.RvThread );
  }

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThreadData* lThreadData = &this->mThreadDatas[i];
    TsmThread*     lTsmThread = lThreadData->TsmThread;

    if ( NULL != lTsmThread )
    {
      if ( TsmThreadState_WAITING_RV == lThreadData->State )
      {
        TsmSvrChecker_TxtInt( "Interrupt T_", lTsmThread->mSequence, st );
        Tsm_Lgl( 7, st );
        lThreadData->State = TsmThreadState_RUNNING;
        lThreadData->bInterrupt = TRUE;
        lThreadData->RvThread->Interrupt( lThreadData->RvThread );
      }
    }
  }
  TsmSvrChecker_CheckAllThreadsWaiting( this );

  this->CbThreadData.TsmThread->TsmReset2( this->CbThreadData.TsmThread );

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      lTsmThread->TsmReset2( lTsmThread );
    }
  }

  for( i=0; i<this->Classes.Number; i++ )
  {
    TsmClass* cl = (TsmClass*) TsmArray_At( &this->Classes, i );
    cl->TsmReset1( cl );
  }
  TsmSvrChecker_NativesReset( this );
}

void TsmSvrChecker_Free( TsmSvrChecker* this )
{
  int i;

  this->CbThreadData.TsmThread->Halt( this->CbThreadData.TsmThread );

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      lTsmThread->Halt( lTsmThread );
    }
  }
}

void TsmSvrChecker_Pause( TsmThreadData* oThreadData )
{
  TsmThread_TriggerWait( oThreadData->TsmThread );
}

void TsmSvrChecker_Continue( TsmThreadData* oThreadData )
{
  oThreadData->State = TsmThreadState_RUNNING;
  oThreadData->TsmThread->TriggerNotify( oThreadData->TsmThread );

  OSAL_ThreadYield();
}

void TsmSvrChecker_ReplayNative( TsmSvrChecker* this, TsmScenItem* oItem )
{
  char* st = Tsm_Malloc( TSM_NCH_FULLNAME );

  sprintf( st, "Natv: %d @ ", oItem->Index );
  oItem->Receiver->FullName( (TsmBase*) oItem->Receiver, st+strlen(st) );
  Tsm_Lgl( 6, st );
  Tsm_Free( st );

  if ( oItem->Index < 0 )
  { /*
     * Client native.
     * Executed by CbThread.
     */
    TsmTrigger* lTrigger = TsmTrigger_Ctor( -oItem->Index );
    TsmEnvelope lEnvelope;

    TsmEnvelope_Ftor( &lEnvelope, lTrigger, oItem->Receiver );
    this->CbThreadData.TsmThread->NativeCall( this->CbThreadData.TsmThread, &lEnvelope );
    TsmSvrChecker_Continue( &this->CbThreadData );
  }
  else
  { /*
     * Server native.
     * Executed by Supervisor.
     */
    oItem->Receiver->ExecuteNative(
      oItem->Receiver, oItem->Index );
  }
}

/*
 * The state changes :
 *     Item->ThreadData->State = TsmThreadState_RUNNING;
 * are needed to force the Supervisor to wait for the threads.
 */
void TsmSvrChecker_ReplayScenItem( TsmSvrChecker* this, TsmScenItem* Item )
{
  switch ( Item->Kind )
  {
  case TsmScenItemKind_RELEASE_CB_MSG :
    TsmAssert( TsmThreadData_HasMessage(Item->ThreadData) );
    TsmSvrChecker_Continue( Item->ThreadData );
    break;
  case TsmScenItemKind_RELEASE_CB_RV2 :
    TsmAssert( Item->ThreadData->State == TsmThreadState_WAITING_RV2 );
    TsmSvrChecker_Continue( Item->ThreadData );
    break;
  case TsmScenItemKind_RELEASE_MSG :
    TsmAssert( TsmThreadData_HasMessage(Item->ThreadData) );
    TsmSvrChecker_Continue( Item->ThreadData );
    break;
  case TsmScenItemKind_RELEASE_RV2 :
    TsmAssert( ( Item->ThreadData->State & TsmThreadState_WAITING_RV ) != 0 );
    TsmSvrChecker_Continue( Item->ThreadData );
    break;
  case TsmScenItemKind_NATIVE :
    TsmSvrChecker_ReplayNative( this, Item );
    break;
  case TsmScenItemKind_CHOICE :
    TsmRecorder_Enable();
    TsmRecorder_Show( "**** BUG " );
    TsmException_Quit( "TismBUG: Handled by 'ChoiceMax'" );
    break;
  case TsmScenItemKind_QO :
    TsmRecorder_Enable();
    TsmException_Quit( "TismBUG" );
    break;
  }
}

void TsmSvrChecker_Replay( TsmSvrChecker* this )
{
  int           i;
  TsmScenItem*  Si;

  for( i=0; i<this->Classes.Number; i++ )
  {
    TsmClass* cl = (TsmClass*) TsmArray_At( &this->Classes, i );
    cl->TsmReset2( cl );
  }

  this->CurrentScenItemIndex = 0;
  TsmRecorder_Reset();

  for ( ; ; )
  {
    TsmSvrChecker_CheckAllThreadsWaiting( this );

    if ( this->CurrentScenItemIndex == this->CurrentScenario->Items.Number )
    { /*
       * Ready replaying, possibly due to Choice items, or
       * empty scenario.
       */
      TsmRecorder_Enable();
      break;
      /****/
    }
    if ( this->CurrentScenItemIndex == this->CurrentScenario->Items.Number-1 )
    { /*
       * The last item is 'new'.
       */
      TsmRecorder_Enable();
    }
    Si = (TsmScenItem*) TsmArray_At( &this->CurrentScenario->Items, this->CurrentScenItemIndex++ );
    TsmSvrChecker_ReplayScenItem( this, Si );
  }
}

TsmScenItem* TsmSvrChecker_GetThreadDataRV2( TsmSvrChecker* this )
{
  int          i;
  TsmScenItem* lScenItem;

  if ( TsmThreadState_WAITING_RV2 == this->CbThreadData.State )
  {
    lScenItem = TsmScenItem_Ctor();
    lScenItem->Kind = TsmScenItemKind_RELEASE_CB_RV2;
    lScenItem->ThreadData = &this->CbThreadData;
    return lScenItem;
  }

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    TsmThread* lTsmThread = this->mThreadDatas[i].TsmThread;
    if ( NULL != lTsmThread )
    {
      if ( TsmThreadState_WAITING_RV2 == this->mThreadDatas[i].State )
      {
        lScenItem = TsmScenItem_Ctor();
        lScenItem->Kind = TsmScenItemKind_RELEASE_RV2;
        lScenItem->ThreadData = &this->mThreadDatas[i];
        return lScenItem;
      }
    }
  }

  return NULL;
}

void TsmSvrChecker_ReplayExtend( TsmSvrChecker* this, TsmScenario* oScenario )
{
  TsmScenItem* lScenItem;

  while ( NULL != (lScenItem = TsmSvrChecker_GetThreadDataRV2( this )) )
  {
    TsmScenario_Add( oScenario, lScenItem );
    this->CurrentScenItemIndex++;
    TsmSvrChecker_ReplayScenItem( this, lScenItem );
    TsmSvrChecker_CheckAllThreadsWaiting( this );
  }
}

void TsmSvrChecker_AddToCurrentScenario( TsmSvrChecker* this, TsmArray* oScenItems )
{
  int          i;
  TsmScenItem* lScenItem;
  TsmScenario* lNewScenario;

  for( i=0; i<oScenItems->Number; i++ )
  {
    lScenItem = (TsmScenItem*) TsmArray_At( oScenItems, i );
    lNewScenario = TsmScenario_Extend( this->CurrentScenario, lScenItem );
    TsmStackList_Push( &this->FutureScenarios, (TsmStackEntry *) lNewScenario );
  }
}

void TsmSvrChecker_AddScenarioNatives( TsmSvrChecker* this, TsmArray* oScenItems, TsmBool bClient )
{
  TsmClass*       Receiver;
  TsmNativeLists* lNativeLists;
  TsmListItem*    pLI;
  int             i;

  TsmMutex_WaitOne( &this->LockNatives );

  for ( i=0; i<this->Classes.Number; i++ )
  {
    Receiver = (TsmClass*) TsmArray_At( &this->Classes, i );
    lNativeLists = &Receiver->Natives;

    pLI = (TsmListItem*) TsmList_First( &lNativeLists->ServerNatives );
    while ( pLI != NULL )
    {
      TsmScenItem* lScenItem = TsmScenItem_Ctor();
      lScenItem->Kind = TsmScenItemKind_NATIVE;
      lScenItem->Index = (int) TsmListItem_Value( pLI );
      lScenItem->Receiver = Receiver;
      TsmArray_Add( oScenItems, lScenItem );

      pLI = TsmListItem_Next( pLI );
    }

    if ( bClient )
    {
      pLI = (TsmListItem*) TsmList_First( &lNativeLists->ClientNatives );
      while ( pLI != NULL )
      {
        TsmScenItem* lScenItem = TsmScenItem_Ctor();
        lScenItem->Kind = TsmScenItemKind_NATIVE;
        lScenItem->Index = (int) TsmListItem_Value( pLI );
        lScenItem->Receiver = Receiver;
        TsmArray_Add( oScenItems, lScenItem );

        pLI = TsmListItem_Next( pLI );
      }
    }
  }
  TsmMutex_Release( &this->LockNatives );
}

void TsmSvrChecker_AddScenarioCbThread( TsmSvrChecker* this, TsmArray* oScenItems )
{
  TsmIntQueue* lTsmIntQueue;
  TsmScenItem* lScenItem;
  
  lTsmIntQueue = this->CbThreadData.TsmThread->GetIntQueue(this->CbThreadData.TsmThread);
  if ( lTsmIntQueue->Number( (TsmQueue*) lTsmIntQueue ) == 0 )
  {
    return;
  }

  if ( this->CbThreadData.State == TsmThreadState_WAITING_RV )
  {
    return;
  }
  TsmAssert( this->CbThreadData.State == TsmThreadState_WAITING_MSG );

  lScenItem = TsmScenItem_Ctor();
  lScenItem->Kind = TsmScenItemKind_RELEASE_CB_MSG;
  lScenItem->ThreadData = &this->CbThreadData;
  TsmArray_Add( oScenItems, lScenItem );
}

void TsmSvrChecker_AddScenarioThreads( TsmSvrChecker* this, TsmArray* oScenItems )
{
  int             i;
  TsmThreadData*  lThreadData;
  TsmThread*      lTsmThread;

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    lThreadData = &this->mThreadDatas[i];
    lTsmThread = lThreadData->TsmThread;
    if ( NULL != lTsmThread )
    {
      if ( TsmThreadState_WAITING_MSG == lThreadData->State )
      {
        if ( lTsmThread->HasMessage( lTsmThread ) )
        {
          TsmScenItem* lScenItem = TsmScenItem_Ctor();
          lScenItem->Kind = TsmScenItemKind_RELEASE_MSG;
          lScenItem->ThreadData = lThreadData;
          TsmArray_Add( oScenItems, lScenItem );
        }
      }
    }
  }
}

int TsmSvrChecker_GetRandomInt( int iMax )
{
  return rand() % iMax;
}

void TsmSvrChecker_GetDuration( long duration, TsmStr sout )
{
  long s;
  long m;

  s = duration % 60;
  duration /= 60;
  m = duration % 60;
  duration /= 60;

  sprintf( sout, "%02ld:%02ld:%02ld", duration, m, s );
}

void TsmSvrChecker_TxtInt( char* s, int k, TsmStr sout )
{
  sprintf( sout, "%s%d", s, k );
}

void TsmSvrChecker_TxtIntLog( char* s, int k )
{
  char st[ 128 ];

  sprintf( st, "%s%d", s, k );
  Tsm_Log( st );
}
