/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmSvrChecker.h
 * version 1
 */
#ifndef TSM_SVRCHECKER_H
#define TSM_SVRCHECKER_H

#ifdef __cplusplus
extern "C" {
#endif

#include <time.h>
#include "OSAL_Gen.h"
#include "TsmTriggerSem.h"
#include "TsmSvrHelpers.h"
#include "TsmMutex.h"
#include "TsmArray.h"
#include "TsmMapString.h"
#include "TsmStackList.h"
#include "TsmGen.h"
#include "TsmThread.h"
#include "TsmThreadType.h"
#include "TsmThreadState.h"
#include "TsmClass.h"

typedef struct TsmSvrCheckerStruct
{
  TsmTriggerSem  mTriggerSem;
  TsmThreadData  MyThreadData;
  TsmThreadData  CbThreadData;
  TsmThreadData  mThreadDatas[ OSAL_NOF_THREADS+1 ];

  TsmMutex       LockNatives;
  TsmMapString   Snapshots;
  TsmArray       Classes;
  TsmStackList   PastScenarios;
  TsmStackList   FutureScenarios;
  TsmScenario*   CurrentScenario;
  int            CurrentScenItemIndex;
  TsmBool        bQueueOverflow;

  int            NumberOfDeadlocks;
  int            NumberOfLivelocks;
  int            NumberOfCycles;
  int            NumberOfScenarios;
  long           NumberOfLoops;
  time_t         StartTime;
  time_t         StopTime;
} TsmSvrChecker;

TsmSvrChecker*  TsmSvrChecker_Instance( void );
void            TsmSvrChecker_Cleanup( void );
TsmSvrChecker*  TsmSvrChecker_Ctor( void );
void            TsmSvrChecker_Ftor( TsmSvrChecker* this );
void            TsmSvrChecker_Gtor( TsmSvrChecker* this );
void            TsmSvrChecker_Dtor( TsmSvrChecker* this );
TsmArray*       TsmSvrChecker_GetClasses( TsmSvrChecker* this );

int     TsmSvrChecker_Register( TsmSvrChecker* this, TsmThread* oTsmThread, TsmThreadType eType, OSAL_Handle hThreadBid );
void    TsmSvrChecker_SetState( TsmSvrChecker* this, OSAL_Handle ThreadBid, TsmThreadState eState );
void    TsmSvrChecker_SetStateRv( TsmSvrChecker* this, OSAL_Handle ThreadBid, TsmThread* oThread );
void    TsmSvrChecker_NativesReset( TsmSvrChecker* this );
void    TsmSvrChecker_NativeEntry( TsmSvrChecker* this, TsmClass* oOwner, const int aNatives[], int NrNatives );
void    TsmSvrChecker_NativeExit( TsmSvrChecker* this, TsmClass* oOwner, const int aNatives[], int NrNatives );
void    TsmSvrChecker_AddQueueOverflow( TsmSvrChecker* this );
TsmBool TsmSvrChecker_ChoiceOKVerify( TsmSvrChecker* this, int MaxNrChoices, int ChoiceIndex, char* sGuard );
void    TsmSvrChecker_Verify( TsmSvrChecker* this );
TsmBool TsmSvrChecker_ChoiceOKSimulate( TsmSvrChecker* this, int MaxNrChoices, int ChoiceIndex );
void    TsmSvrChecker_Simulate( TsmSvrChecker* this );
void    TsmSvrChecker_Summary( TsmSvrChecker* this, TsmBool bVerify );
void    TsmSvrChecker_GetCurrentSituation( TsmSvrChecker* this, TsmStr sout );
void    TsmSvrChecker_SnappedClasses( TsmSvrChecker* this, TsmStr sout );
void    TsmSvrChecker_SnappedThreads( TsmSvrChecker* this, TsmStr sout );
void    TsmSvrChecker_GetThreadStates( TsmSvrChecker* this, char* sHeader, TsmStr sout );

#ifdef __cplusplus
}
#endif 
#endif
